package com.example.camera;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.provider.MediaStore;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Toast;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.FileProvider;

public class MainActivity extends AppCompatActivity {

    public static final byte HEADER_TEXT = 0x01;
    public static final byte HEADER_IMAGE = 0x02;

    private static final String IP = "172.21.17.238";
    private static final int PORT = 8000;

    private Socket socket;
    private OutputStream os;

    private static final int REQUEST_IMAGE_CAPTURE = 40;
    private static final int REQUEST_PERMISSION = 50;

    private static final int HANDLER_MSG_EXIT = 999;
    private static final int HANDLER_MSG_CONNECT = 100;

    private Button connectButton;
    private LinearLayout msgLayout;
    private EditText msgEdit;
    private Button sendMsgButton;

    private Button sendCameraButton;


    @RequiresApi(api = Build.VERSION_CODES.M)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        if (checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_DENIED) {
            requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, REQUEST_PERMISSION);
        }
        init();
    }
    private byte[] getDataLength(int length) {
        byte[] blen = new byte[4];

        blen[0] |= (byte) ((length & 0xFF000000) >> 24);
        blen[1] |= (byte) ((length & 0xFF0000) >> 16);
        blen[2] |= (byte) ((length & 0xFF00) >> 8);
        blen[3] |= (byte) (length & 0xFF);

        return blen;

    }
    private byte[] objectToByte(byte header, byte[] data) {

        byte[] result = null;

        int size = 1 + 4 + data.length;

        result = new byte[size];

        result[0] = header;

        byte[] dataLen = getDataLength(data.length);

        System.arraycopy(dataLen, 0, result, 1, dataLen.length);

        System.arraycopy(data, 0, result, 5, data.length);

        return result;
    }

    private void init() {
        connectButton = findViewById(R.id.connect_server_button);
        connectButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                connectServer();
            }
        });

        msgLayout = findViewById(R.id.msg_layout);
        msgLayout.setVisibility(View.GONE);

        msgEdit = findViewById(R.id.msg_edit);
        sendMsgButton = findViewById(R.id.send_msg_button);
        sendCameraButton = findViewById(R.id.send_camera_button);
        sendCameraButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                goCamera();
            }
        });
    }
    private void goCamera() {
        Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
        try {
            //String path = Environment.getExternalStorageDirectory().getAbsolutePath() + "/test.jpg";
            String path = getFilesDir().getPath()+"/test.jpg";
            File file = new File(path);
            if (!file.isFile()) {
                file.createNewFile();
            }

            Uri uri = FileProvider.getUriForFile(this, "com.example.fireproof", file);

            intent.putExtra(MediaStore.EXTRA_OUTPUT, uri);
            startActivityForResult(intent, REQUEST_IMAGE_CAPTURE);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == REQUEST_PERMISSION) {
            for (int result : grantResults) {
                if (result == PackageManager.PERMISSION_DENIED) {
                    Toast.makeText(this, "권한 줘라.", Toast.LENGTH_SHORT).show();
                    finish();
                }
            }
        }
    }
    private void connectServer() {
        Log.e("test", "connectServer");
        new ServerConnectThread().start();
    }

    class ServerConnectThread extends Thread {
        @Override
        public void run() {
            try {
                socket = new Socket();
                SocketAddress addr = new InetSocketAddress(IP, PORT);
                socket.connect(addr);
                os = socket.getOutputStream();
                handler.sendEmptyMessage(HANDLER_MSG_CONNECT);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private Handler handler = new Handler() {
        @Override
        public void handleMessage(@NonNull Message msg) {
            super.handleMessage(msg);

            int what = msg.what;
            if (what == HANDLER_MSG_CONNECT) {
                completeConnect();
            } else if (what == HANDLER_MSG_EXIT) {
                connectButton.setEnabled(true);
                msgLayout.setVisibility(View.GONE);
            }
        }
    };

    private void completeConnect() {
        connectButton.setEnabled(false);
        msgLayout.setVisibility(View.VISIBLE);
        sendMsgButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                sendMsg();
            }
        });
    }
    private void sendMsg() {
        final String msg = msgEdit.getText().toString();
        msgEdit.setText("");

        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    if (msg.equals("exit")) {
                        os.close();
                        socket.close();
                        handler.sendEmptyMessage(HANDLER_MSG_EXIT);
                    } else {
                        byte[] data = objectToByte(HEADER_TEXT, msg.getBytes("utf-8"));
                        os.write(data, 0, data.length);
                        os.flush();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }).start();
    }
    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == REQUEST_IMAGE_CAPTURE) {
            if (resultCode == RESULT_OK) {
                Toast.makeText(this, "촬영 완료", Toast.LENGTH_SHORT).show();
                sendImage();
            }
        }
    }
    private void sendImage() {

        String path = getFilesDir().getPath()+"/test.jpg";
        File file = new File(path);

        final byte[] imageByteArr = new byte[(int) file.length()];

        try {
            FileInputStream fis = new FileInputStream(file);
            fis.read(imageByteArr);
            fis.close();

            new Thread(new Runnable() {
                @Override
                public void run() {
                    try {
                        byte[] data = objectToByte(HEADER_IMAGE, imageByteArr);
                        os.write(data);
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }).start();

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}